# Set system specific flags based on OS and arch.
#
# Can pass in CFlags to use. E.g (-Wconversion -Wextra -Wnon-exist) or call without CFlags to
# use the default list of CFlags. Once done this will set all of the supported cflags.

INCLUDE (DetectPlatform)
INCLUDE (SystemExtensions)
INCLUDE (SetBuildTypeFlags)

MACRO (M_SET_SYSTEM_TYPE_FLAGS)
	M_SET_BUILD_TYPE_FLAGS (${ARGN})
	M_CHECK_SYSTEM_EXTENSIONS ()

	# Ensure if we can set PIC it's set no matter what.
	IF (UNIX AND CMAKE_SYSTEM_PROCESSOR MATCHES "x86_64")
		ADD_DEFINITIONS (-fPIC)
	ENDIF ()

	IF ("${OSTYPE}" STREQUAL "aix")
		IF ("${OSBITS}" STREQUAL "32")
			SET (CMAKE_C_FLAGS "${CMAKE_C_FLAGS} -maix32")
			IF (NOT "${IS_DEBUG}")
				SET (CMAKE_C_FLAGS "${CMAKE_C_FLAGS}  -mcpu=power3")
			ENDIF ()
			SET (CMAKE_EXE_LINKER_FLAGS "${CMAKE_EXE_LINKER_FLAGS} -b32")
			SET (CMAKE_SHARED_LINKER_FLAGS "${CMAKE_SHARED_LINKER_FLAGS} -b32")
			SET (CMAKE_MODULE_LINKER_FLAGS "${CMAKE_MODULE_LINKER_FLAGS} -b32")
			SET(CMAKE_C_ARCHIVE_CREATE "${CMAKE_C_ARCHIVE_CREATE} -X 32 -r -c")
			#NM="nm -X 32 -B"
		ELSE ()
			SET (CMAKE_C_FLAGS "${CMAKE_C_FLAGS} -maix64")
			SET (CMAKE_EXE_LINKER_FLAGS "${CMAKE_EXE_LINKER_FLAGS} -b64")
			SET (CMAKE_SHARED_LINKER_FLAGS "${CMAKE_SHARED_LINKER_FLAGS} -b64")
			SET (CMAKE_MODULE_LINKER_FLAGS "${CMAKE_MODULE_LINKER_FLAGS} -b64")
			SET(CMAKE_C_ARCHIVE_CREATE "${CMAKE_C_ARCHIVE_CREATE} -X 64 -r -c")
			#NM="nm -X 64 -B"
		ENDIF ()
	ELSEIF ("${OSTYPE}" STREQUAL "linux")
		IF ("${OSARCH}" STREQUAL "x86")
			IF (NOT "${IS_DEBUG}")
				SET (CMAKE_EXE_LINKER_FLAGS "${CMAKE_EXE_LINKER_FLAGS} -Wl,--hash-style=both")
				SET (CMAKE_SHARED_LINKER_FLAGS "${CMAKE_SHARED_LINKER_FLAGS} -Wl,--hash-style=both")
				SET (CMAKE_MODULE_LINKER_FLAGS "${CMAKE_MODULE_LINKER_FLAGS} -Wl,--hash-style=both")
			ENDIF ()
		ENDIF ()
	ELSEIF ("${OSTYPE}" STREQUAL "freebsd")
		IF ("${OSBITS}" STREQUAL "32")
			IF (NOT "${IS_DEBUG}")
				SET (CMAKE_C_FLAGS "${CMAKE_C_FLAGS} -march=pentium-mmx -mtune=i686")
			ENDIF ()
		ENDIF ()
	ELSEIF ("${OSTYPE}" STREQUAL "openbsd")
		IF (NOT "${IS_DEBUG}")
			SET (CMAKE_C_FLAGS "${CMAKE_C_FLAGS} -march=pentium-mmx")
		ENDIF ()
	ELSEIF ("${OSTYPE}" STREQUAL "solaris")
		IF ("${OSBITS}" STREQUAL "32")
			SET (CMAKE_C_FLAGS "${CMAKE_C_FLAGS} -m32 -static-libgcc")
			IF ("${OSARCH}" STREQUAL "x86")
				IF (NOT "$IS_DEBUG")
					SET (CMAKE_C_FLAGS "${CMAKE_C_FLAGS} -march=pentium-mmx -mtune=i686")
				ENDIF ()
			ENDIF ()
		ELSEIF ("${OSBITS}" STREQUAL "64")
			SET (CMAKE_C_FLAGS "${CMAKE_C_FLAGS} -m64 -static-libgcc")
		ENDIF ()
		SET (CMAKE_EXE_LINKER_FLAGS "${CMAKE_EXE_LINKER_FLAGS} -lc")
		SET (CMAKE_SHARED_LINKER_FLAGS "${CMAKE_SHARED_LINKER_FLAGS} -lc")
		SET (CMAKE_MODULE_LINKER_FLAGS "${CMAKE_MODULE_LINKER_FLAGS} -lc")
	ELSEIF (IOSSIM)
		SET (CMAKE_C_FLAGS "${CMAKE_C_FLAGS} -DIOS=1 -DIOSSIM=1 -fobjc-arc -mios-simulator-version-min=7.0")
	ELSEIF (IOS)
		SET (CMAKE_C_FLAGS "${CMAKE_C_FLAGS} -DIOS=1 -fobjc-arc -fembed-bitcode -miphoneos-version-min=6.0")
	ELSEIF ("${OSTYPE}" STREQUAL "macosx")
		# Try to use 10.6 SDK as that's the oldest version we can support.
		# If it can't detect the 10.6 SDK it should try to use the newerest
		# SDK on the system (even if it's in XCode and not linked into /Developer.
		# NOTE: If we chnage to target a newer SDK explicitly we need to
		#       1) Symlink the sdk to /Developer (or set the appropriate path).
		#       2) Set appropriate variables below.
		#       3) Realize that the uname -r gives the kernel version which doens't
		#          necessarily align wih the SDK version avaliable. For example
		#          macdev right now shows 11.2.0 and had 10.6 and 10.7 SDKs. macdev
		#          is actually running OS X 10.7.
		EXEC_PROGRAM ("uname -r" OUTPUT_VARIABLE OSX_KERN_VER)
		IF ("${OSX_KERN_VER}" MATCHES "11[.].*" AND EXISTS "/Developer/SDKs/MacOSX10.6.sdk")
			SET (CMAKE_OSX_SYSROOT "/Developer/SDKs/MacOSX10.6.sdk")
			SET (CMAKE_OSX_DEPLOYMENT_TARGET "10.6")
		ENDIF ()
		# We only support x86 builds right now.
		#IF ("${OSBITS}" STREQUAL "64" OR "${OSARCH}" STREQUAL "x64")
		#	SET (OSBITS "32")
		#	SET (OSARCH "x86")
		#ENDIF ()
		IF ("${OSBITS}" STREQUAL "32" OR "${OSARCH}" STREQUAL "x86")
			SET (CMAKE_C_FLAGS "${CMAKE_C_FLAGS} -arch i386")
		ELSE ()
			SET (CMAKE_C_FLAGS "${CMAKE_C_FLAGS} -arch x86_64")
		ENDIF ()
		# If a deployment target is set use it otherwise we don't set it and let the system decide.
		IF (NOT "${CMAKE_OSX_SYSROOT}" STREQUAL "" AND NOT "${CMAKE_OSX_DEPLOYMENT_TARGET}" STREQUAL "")
			SET (CMAKE_C_FLAGS "${CMAKE_C_FLAGS} -isysroot ${CMAKE_OSX_SYSROOT} -mmacosx-version-min=${CMAKE_OSX_DEPLOYMENT_TARGET}")
			SET (CMAKE_EXE_LINKER_FLAGS "${CMAKE_EXE_LINKER_FLAGS} -Wl,-syslibroot,${CMAKE_OSX_SYSROOT}")
			SET (CMAKE_SHARED_LINKER_FLAGS "${CMAKE_SHARED_LINKER_FLAGS} -Wl,-syslibroot,${CMAKE_OSX_SYSROOT}")
			SET (CMAKE_MODULE_LINKER_FLAGS "${CMAKE_MODULE_LINKER_FLAGS} -Wl,-syslibroot,${CMAKE_OSX_SYSROOT}")
		ENDIF ()
	ENDIF ()
ENDMACRO ()
